@@ -1,7 +1,12 @@
 """Common functionality for backends."""
 
 import abc
+import os.path
+from datetime import datetime, timezone
+from glob import glob
 from annif import logger
+from annif.corpus import TruncatingDocumentCorpus
+from annif.exception import ConfigurationException
 
 
 class AnnifBackend(metaclass=abc.ABCMeta):
@@ -11,7 +16,8 @@ class AnnifBackend(metaclass=abc.ABCMeta):
     name = None
     needs_subject_index = False
 
-    DEFAULT_PARAMETERS = {'limit': 100}
+    DEFAULT_PARAMETERS = {'limit': 100,
+                          'input_limit': 0}
 
     def __init__(self, backend_id, config_params, project):
         """Initialize backend with specific parameters. The
@@ -32,12 +38,33 @@ class AnnifBackend(metaclass=abc.ABCMeta):
         params.update(self.config_params)
         return params
 
+    @property
+    def is_trained(self):
+        return bool(glob(os.path.join(self.datadir, '*')))
+
+    @property
+    def modification_time(self):
+        mtimes = [datetime.utcfromtimestamp(os.path.getmtime(p))
+                  for p in glob(os.path.join(self.datadir, '*'))]
+        most_recent = max(mtimes, default=None)
+        if most_recent is None:
+            return None
+        return most_recent.replace(tzinfo=timezone.utc)
+
     def _get_backend_params(self, params):
         backend_params = dict(self.params)
         if params is not None:
             backend_params.update(params)
         return backend_params
 
+    def _validate_input_limit(self, input_limit):
+        input_limit = int(input_limit)
+        if input_limit >= 0:
+            return input_limit
+        else:
+            raise ConfigurationException(
+                'input_limit can not be negative', backend_id=self.backend_id)
+
     def _train(self, corpus, params):
         """This method can be overridden by backends. It implements
         the train functionality, with pre-processed parameters."""
@@ -46,6 +73,9 @@ class AnnifBackend(metaclass=abc.ABCMeta):
     def train(self, corpus, params=None):
         """Train the model on the given document or subject corpus."""
         beparams = self._get_backend_params(params)
+        input_limit = self._validate_input_limit(beparams['input_limit'])
+        if input_limit != 0:
+            corpus = TruncatingDocumentCorpus(corpus, input_limit)
         return self._train(corpus, params=beparams)
 
     def initialize(self):
@@ -64,6 +94,9 @@ class AnnifBackend(metaclass=abc.ABCMeta):
         represented as a list of SubjectSuggestion objects."""
         beparams = self._get_backend_params(params)
         self.initialize()
+        input_limit = self._validate_input_limit(beparams['input_limit'])
+        if input_limit != 0:
+            text = text[:input_limit]
         return self._suggest(text, params=beparams)
 
     def debug(self, message):
@@ -91,4 +124,7 @@ class AnnifLearningBackend(AnnifBackend):
     def learn(self, corpus, params=None):
         """Further train the model on the given document or subject corpus."""
         beparams = self._get_backend_params(params)
+        input_limit = self._validate_input_limit(beparams['input_limit'])
+        if input_limit != 0:
+            corpus = TruncatingDocumentCorpus(corpus, input_limit)
         return self._learn(corpus, params=beparams)
