@@ -5,7 +5,7 @@ import os.path
 import annif.util
 from annif.suggestion import SubjectSuggestion, ListSuggestionResult
 from annif.exception import NotInitializedException, NotSupportedException
-import fastText
+import fasttext
 from . import backend
 from . import mixins
 
@@ -30,7 +30,8 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
         'minn': int,
         'maxn': int,
         'thread': int,
-        't': float
+        't': float,
+        'pretrainedVectors': str
     }
 
     DEFAULT_PARAMETERS = {
@@ -52,12 +53,23 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
         params.update(self.DEFAULT_PARAMETERS)
         return params
 
+    @staticmethod
+    def _load_model(path):
+        # monkey patch fasttext.FastText.eprint to avoid spurious warning
+        # see https://github.com/facebookresearch/fastText/issues/1067
+        orig_eprint = fasttext.FastText.eprint
+        fasttext.FastText.eprint = lambda x: None
+        model = fasttext.load_model(path)
+        # restore the original eprint
+        fasttext.FastText.eprint = orig_eprint
+        return model
+
     def initialize(self):
         if self._model is None:
             path = os.path.join(self.datadir, self.MODEL_FILE)
             self.debug('loading fastText model from {}'.format(path))
             if os.path.exists(path):
-                self._model = fastText.load_model(path)
+                self._model = self._load_model(path)
                 self.debug('loaded model {}'.format(str(self._model)))
                 self.debug('dim: {}'.format(self._model.get_dimension()))
             else:
@@ -74,15 +86,20 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
         subject_id = int(labelnum)
         return self.project.subjects[subject_id]
 
-    def _write_train_file(self, doc_subjects, filename):
+    def _write_train_file(self, corpus, filename):
         with open(filename, 'w', encoding='utf-8') as trainfile:
-            for doc, subject_ids in doc_subjects.items():
+            for doc in corpus.documents:
+                text = self._normalize_text(doc.text)
+                if text == '':
+                    continue
+                subject_ids = [self.project.subjects.by_uri(uri)
+                               for uri in doc.uris]
                 labels = [self._id_to_label(sid) for sid in subject_ids
                           if sid is not None]
                 if labels:
-                    print(' '.join(labels), doc, file=trainfile)
+                    print(' '.join(labels), text, file=trainfile)
                 else:
-                    self.warning('no labels for document "{}"'.format(doc))
+                    self.warning(f'no labels for document "{doc.text}"')
 
     def _normalize_text(self, text):
         return ' '.join(self.project.analyzer.tokenize_words(text))
@@ -90,16 +107,7 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
     def _create_train_file(self, corpus):
         self.info('creating fastText training file')
 
-        doc_subjects = collections.defaultdict(set)
-
-        for doc in corpus.documents:
-            text = self._normalize_text(doc.text)
-            if text == '':
-                continue
-            doc_subjects[text] = [self.project.subjects.by_uri(uri)
-                                  for uri in doc.uris]
-
-        annif.util.atomic_save(doc_subjects,
+        annif.util.atomic_save(corpus,
                                self.datadir,
                                self.TRAIN_FILE,
                                method=self._write_train_file)
@@ -112,7 +120,7 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
                   for param, val in params.items()
                   if param in self.FASTTEXT_PARAMS}
         self.debug('Model parameters: {}'.format(params))
-        self._model = fastText.train_supervised(trainpath, **params)
+        self._model = fasttext.train_supervised(trainpath, **params)
         self._model.save_model(modelpath)
 
     def _train(self, corpus, params):
@@ -151,4 +159,4 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
                 label=subject[1],
                 notation=subject[2],
                 score=score / len(chunktexts)))
-        return ListSuggestionResult(results, self.project.subjects)
+        return ListSuggestionResult(results)
