@@ -14,7 +14,6 @@ from tensorflow.keras.models import Model, load_model
 from tensorflow.keras.utils import Sequence
 import tensorflow.keras.backend as K
 import annif.corpus
-import annif.project
 import annif.util
 from annif.exception import NotInitializedException
 from annif.suggestion import VectorSuggestionResult
@@ -77,7 +76,7 @@ class LMDBSequence(Sequence):
 
 class NNEnsembleBackend(
         backend.AnnifLearningBackend,
-        ensemble.EnsembleBackend):
+        ensemble.BaseEnsembleBackend):
     """Neural network ensemble backend that combines results from multiple
     projects"""
 
@@ -99,12 +98,12 @@ class NNEnsembleBackend(
     _model = None
 
     def default_params(self):
-        params = {}
-        params.update(super().default_params())
+        params = backend.AnnifBackend.DEFAULT_PARAMETERS.copy()
         params.update(self.DEFAULT_PARAMETERS)
         return params
 
     def initialize(self):
+        super().initialize()
         if self._model is not None:
             return  # already initialized
         model_filename = os.path.join(self.datadir, self.MODEL_FILE)
@@ -116,12 +115,13 @@ class NNEnsembleBackend(
         self._model = load_model(model_filename)
 
     def _merge_hits_from_sources(self, hits_from_sources, params):
-        score_vector = np.array([hits.vector * weight
-                                 for hits, weight in hits_from_sources],
+        score_vector = np.array([hits.as_vector(subjects) * weight
+                                 for hits, weight, subjects
+                                 in hits_from_sources],
                                 dtype=np.float32)
         results = self._model.predict(
             np.expand_dims(score_vector.transpose(), 0))
-        return VectorSuggestionResult(results[0], self.project.subjects)
+        return VectorSuggestionResult(results[0])
 
     def _create_model(self, sources):
         self.info("creating NN ensemble model")
@@ -159,7 +159,7 @@ class NNEnsembleBackend(
 
     def _corpus_to_vectors(self, corpus, seq):
         # pass corpus through all source projects
-        sources = [(annif.project.get_project(project_id), weight)
+        sources = [(self.project.registry.get_project(project_id), weight)
                    for project_id, weight
                    in annif.util.parse_sources(self.params['sources'])]
 
@@ -167,7 +167,8 @@ class NNEnsembleBackend(
             doc_scores = []
             for source_project, weight in sources:
                 hits = source_project.suggest(doc.text)
-                doc_scores.append(hits.vector * weight)
+                doc_scores.append(
+                    hits.as_vector(source_project.subjects) * weight)
             score_vector = np.array(doc_scores,
                                     dtype=np.float32).transpose()
             subjects = annif.corpus.SubjectSet((doc.uris, doc.labels))
@@ -182,14 +183,16 @@ class NNEnsembleBackend(
 
     def _fit_model(self, corpus, epochs):
         env = self._open_lmdb(corpus == 'cached')
-        with env.begin(write=True, buffers=True) as txn:
-            seq = LMDBSequence(txn, batch_size=32)
-            if corpus != 'cached':
+        if corpus != 'cached':
+            with env.begin(write=True, buffers=True) as txn:
+                seq = LMDBSequence(txn, batch_size=32)
                 self._corpus_to_vectors(corpus, seq)
-            else:
-                self.info("Reusing cached training data from previous run.")
+        else:
+            self.info("Reusing cached training data from previous run.")
 
-            # fit the model
+        # fit the model using a read-only view of the LMDB
+        with env.begin(buffers=True) as txn:
+            seq = LMDBSequence(txn, batch_size=32)
             self._model.fit(seq, verbose=True, epochs=epochs)
 
         annif.util.atomic_save(
