@@ -1,7 +1,8 @@
 """Annif corpus operations"""
 
 
-from .document import DocumentDirectory, DocumentFile, DocumentList
+from .document import DocumentDirectory, DocumentFile, DocumentList, \
+    TruncatingDocumentCorpus
 from .subject import Subject, SubjectFileTSV
 from .subject import SubjectIndex, SubjectSet
 from .skos import SubjectFileSKOS
@@ -10,4 +11,4 @@ from .combine import CombinedCorpus
 
 __all__ = [DocumentDirectory, DocumentFile, DocumentList, Subject,
            SubjectFileTSV, SubjectIndex, SubjectSet, SubjectFileSKOS,
-           Document, CombinedCorpus]
+           Document, CombinedCorpus, TruncatingDocumentCorpus]
