@@ -1,7 +1,6 @@
 """Unit tests for backends in Annif"""
 
 import pytest
-import logging
 import annif
 import annif.backend
 import annif.corpus
@@ -18,9 +17,10 @@ def test_get_backend_dummy(project):
                        project=project)
     result = dummy.suggest(text='this is some text')
     assert len(result) == 1
-    assert result[0].uri == 'http://example.org/dummy'
-    assert result[0].label == 'dummy'
-    assert result[0].score == 1.0
+    hits = result.as_list(project.subjects)
+    assert hits[0].uri == 'http://example.org/dummy'
+    assert hits[0].label == 'dummy'
+    assert hits[0].score == 1.0
 
 
 def test_learn_dummy(project, tmpdir):
@@ -38,14 +38,16 @@ def test_learn_dummy(project, tmpdir):
 
     result = dummy.suggest(text='this is some text')
     assert len(result) == 1
-    assert result[0].uri == 'http://example.org/key1'
-    assert result[0].label == 'key1'
-    assert result[0].score == 1.0
+    hits = result.as_list(project.subjects)
+    assert hits[0].uri == 'http://example.org/key1'
+    assert hits[0].label == 'key1'
+    assert hits[0].score == 1.0
 
 
 def test_fill_params_with_defaults(project):
     dummy_type = annif.backend.get_backend('dummy')
     dummy = dummy_type(backend_id='dummy', config_params={},
                        project=project)
-    expected_default_params = {'limit': 100}  # From AnnifBackend class
+    expected_default_params = {'limit': 100,
+                               'input_limit': 0}
     assert expected_default_params == dummy.params
