@@ -55,8 +55,8 @@ def test_list_projects_config_path_option_nonexistent():
         annif.cli.cli, ["list-projects", "--projects", "nonexistent.cfg"])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Error: Invalid value for "-p" / "--projects": ' \
-           'File "nonexistent.cfg" does not exist.' in failed_result.output
+    assert "Error: Invalid value for '-p' / '--projects': " \
+           "File 'nonexistent.cfg' does not exist." in failed_result.output
 
 
 def test_show_project():
@@ -71,6 +71,10 @@ def test_show_project():
     assert project_lang.group(1) == 'en'
     access = re.search(r'Access:\s+(.+)', result.output)
     assert access.group(1) == 'hidden'
+    is_trained = re.search(r'Trained:\s+(.+)', result.output)
+    assert is_trained.group(1) == 'True'
+    modification_time = re.search(r'Modification time:\s+(.+)', result.output)
+    assert modification_time.group(1) == 'None'
 
 
 def test_show_project_nonexistent():
@@ -189,8 +193,8 @@ def test_loadvoc_nonexistent_path():
             'loadvoc', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "SUBJECTFILE": ' \
-           'File "nonexistent_path" does not exist.' in failed_result.output
+    assert "Invalid value for 'SUBJECTFILE': " \
+           "File 'nonexistent_path' does not exist." in failed_result.output
 
 
 def test_train(testdatadir):
@@ -269,8 +273,8 @@ def test_train_nonexistent_path():
             'train', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "[PATHS]...": ' \
-           'Path "nonexistent_path" does not exist.' in failed_result.output
+    assert "Invalid value for '[PATHS]...': " \
+           "Path 'nonexistent_path' does not exist." in failed_result.output
 
 
 def test_train_no_path(caplog):
@@ -312,8 +316,8 @@ def test_learn_nonexistent_path():
             'learn', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "[PATHS]...": ' \
-           'Path "nonexistent_path" does not exist.' in failed_result.output
+    assert "Invalid value for '[PATHS]...': " \
+           "Path 'nonexistent_path' does not exist." in failed_result.output
 
 
 def test_suggest():
@@ -413,8 +417,8 @@ def test_index_nonexistent_path():
             'index', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "DIRECTORY": ' \
-           'Directory "nonexistent_path" does not exist.' \
+    assert "Invalid value for 'DIRECTORY': " \
+           "Directory 'nonexistent_path' does not exist." \
            in failed_result.output
 
 
@@ -508,6 +512,79 @@ def test_eval_param(tmpdir):
     assert float(recall.group(1)) == 0.0
 
 
+def test_eval_resultsfile(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+    tmpdir.join('doc3.txt').write('doc3')
+    resultfile = tmpdir.join('results.tsv')
+    result = runner.invoke(
+        annif.cli.cli, [
+            'eval', '--results-file', str(resultfile), 'dummy-en',
+            str(tmpdir)])
+    assert not result.exception
+    assert result.exit_code == 0
+
+    # subject average should equal average of all subject scores in outputfile
+    precision = float(
+        re.search(r'Precision .*subj.*:\s+(\d.\d+)', result.output).group(1))
+    recall = float(
+        re.search(r'Recall .*subj.*:\s+(\d.\d+)', result.output).group(1))
+    f_measure = float(
+        re.search(r'F1 score .*subj.*:\s+(\d.\d+)', result.output).group(1))
+    precision_numerator = 0
+    recall_numerator = 0
+    f_measure_numerator = 0
+    denominator = 0
+    with resultfile.open() as f:
+        header = next(f)
+        assert header.strip('\n') == '\t'.join(['URI',
+                                                'Label',
+                                                'Support',
+                                                'True_positives',
+                                                'False_positives',
+                                                'False_negatives',
+                                                'Precision',
+                                                'Recall',
+                                                'F1_score'])
+        for line in f:
+            assert line.strip() != ''
+            parts = line.split('\t')
+            if parts[1] == 'dummy':
+                assert int(parts[2]) == 1
+                assert int(parts[3]) == 1
+                assert int(parts[4]) == 1
+                assert int(parts[5]) == 0
+            if parts[1] == 'none':
+                assert int(parts[2]) == 1
+                assert int(parts[3]) == 0
+                assert int(parts[4]) == 0
+                assert int(parts[5]) == 1
+            precision_numerator += float(parts[6])
+            recall_numerator += float(parts[7])
+            f_measure_numerator += float(parts[8])
+            denominator += 1
+    assert precision_numerator / denominator == precision
+    assert recall_numerator / denominator == recall
+    assert f_measure_numerator / denominator == f_measure
+
+
+def test_eval_badresultsfile(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+    tmpdir.join('doc3.txt').write('doc3')
+    failed_result = runner.invoke(
+        annif.cli.cli, [
+            'eval', '--results-file', 'newdir/test_file.txt', 'dummy-en',
+            str(tmpdir)])
+    assert failed_result.exception
+    assert failed_result.exit_code != 0
+    assert 'cannot open results-file for writing' in failed_result.output
+
+
 def test_eval_docfile():
     docfile = os.path.join(
         os.path.dirname(__file__),
@@ -535,8 +612,34 @@ def test_eval_nonexistent_path():
             'eval', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "[PATHS]...": ' \
-           'Path "nonexistent_path" does not exist.' in failed_result.output
+    assert "Invalid value for '[PATHS]...': " \
+           "Path 'nonexistent_path' does not exist." in failed_result.output
+
+
+def test_eval_single_process(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+    tmpdir.join('doc3.txt').write('doc3')
+
+    result = runner.invoke(
+        annif.cli.cli, ['eval', '--jobs', '1', 'dummy-en', str(tmpdir)])
+    assert not result.exception
+    assert result.exit_code == 0
+
+
+def test_eval_two_jobs(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+    tmpdir.join('doc3.txt').write('doc3')
+
+    result = runner.invoke(
+        annif.cli.cli, ['eval', '--jobs', '2', 'dummy-en', str(tmpdir)])
+    assert not result.exception
+    assert result.exit_code == 0
 
 
 def test_optimize_dir(tmpdir):
@@ -583,5 +686,64 @@ def test_optimize_nonexistent_path():
             'optimize', 'dummy-fi', 'nonexistent_path'])
     assert failed_result.exception
     assert failed_result.exit_code != 0
-    assert 'Invalid value for "[PATHS]...": ' \
-           'Path "nonexistent_path" does not exist.' in failed_result.output
+    assert "Invalid value for '[PATHS]...': " \
+           "Path 'nonexistent_path' does not exist." in failed_result.output
+
+
+def test_hyperopt_ensemble(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+
+    result = runner.invoke(
+        annif.cli.cli, [
+            'hyperopt', 'ensemble', str(tmpdir)])
+    assert not result.exception
+    assert result.exit_code == 0
+
+    assert re.search(
+        r'sources=dummy-en:0.\d+,dummydummy:0.\d+',
+        result.output) is not None
+
+
+def test_hyperopt_ensemble_resultsfile(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+    resultfile = tmpdir.join('results.tsv')
+
+    result = runner.invoke(
+        annif.cli.cli, [
+            'hyperopt', '--results-file', str(resultfile), 'ensemble',
+            str(tmpdir)])
+    assert not result.exception
+    assert result.exit_code == 0
+
+    with resultfile.open() as f:
+        header = next(f)
+        assert header.strip('\n') == '\t'.join(['trial',
+                                                'value',
+                                                'dummy-en',
+                                                'dummydummy'])
+        for idx, line in enumerate(f):
+            assert line.strip() != ''
+            parts = line.split('\t')
+            assert len(parts) == 4
+            assert int(parts[0]) == idx
+
+
+def test_hyperopt_not_supported(tmpdir):
+    tmpdir.join('doc1.txt').write('doc1')
+    tmpdir.join('doc1.key').write('dummy')
+    tmpdir.join('doc2.txt').write('doc2')
+    tmpdir.join('doc2.key').write('none')
+
+    failed_result = runner.invoke(
+        annif.cli.cli, [
+            'hyperopt', 'tfidf-en', str(tmpdir)])
+    assert failed_result.exception
+    assert failed_result.exit_code != 0
+
+    assert 'Hyperparameter optimization not supported' in failed_result.output
