@@ -1,7 +1,9 @@
 """Unit tests for corpus functionality in Annif"""
 
 import gzip
+import numpy as np
 import annif.corpus
+from annif.corpus import TruncatingDocumentCorpus
 
 
 def test_subjectset_uris():
@@ -47,6 +49,16 @@ def test_subjectset_as_vector(subject_index):
     assert vector.sum() == 1  # only one known subject
 
 
+def test_subjectset_as_vector_destination(subject_index):
+    uris = ['http://www.yso.fi/onto/yso/p10849', 'http://example.org/unknown']
+    labels = ['arkeologit', 'unknown-subject']
+    sset = annif.corpus.SubjectSet((uris, labels))
+    destination = np.zeros(len(subject_index), dtype=np.float32)
+    vector = sset.as_vector(subject_index, destination=destination)
+    assert vector.sum() == 1  # only one known subject
+    assert vector is destination
+
+
 def test_docdir_key(tmpdir):
     tmpdir.join('doc1.txt').write('doc1')
     tmpdir.join('doc1.key').write('key1')
@@ -254,3 +266,11 @@ def test_combinedcorpus(tmpdir):
     combined = annif.corpus.CombinedCorpus([corpus1, corpus2])
 
     assert len(list(combined.documents)) == 6
+
+
+def test_truncatedcorpus(document_corpus):
+    truncated_corpus = TruncatingDocumentCorpus(document_corpus, 3)
+    for truncated_doc, doc in zip(truncated_corpus.documents,
+                                  document_corpus.documents):
+        assert len(truncated_doc.text) == 3
+        assert truncated_doc.text == doc.text[:3]
